RAW = {
    'status': {
        'hrDeviceStatus': 2,
        'status1': '  Sleeping...   ',
        'status2': '                ',
        'status3': '',
        'status4': ''
    },
    'identity': {
        'model_name': 'M2070 Series',
        'host_name': 'SEC811119110648',
        'location': '',
        'serial_num': 'ZFAYB8KGGGG1GZP',
        'ip_addr': '192.168.0.0',
        'ipv6_link_addr': '',
        'mac_addr': '84:25:00:00:06:48',
        'admin_email': '',
        'admin_name': '',
        'admin_phone': '',
        'customer_support': ''
    },
    'toner_black': {
        'opt': 1,
        'remaining': 58,
        'cnt': 229,
        'newError': ''
    },
    'toner_cyan': {
        'opt': 0,
        'remaining': 0,
        'cnt': 0,
        'newError': ''
    },
    'toner_magenta': {
        'opt': 0,
        'remaining': 0,
        'cnt': 0,
        'newError': ''
    },
    'toner_yellow': {
        'opt': 0,
        'remaining': 0,
        'cnt': 0,
        'newError': ''
    },
    'drum_black': {
        'opt': 0,
        'remaining': 0,
        'newError': ''
    },
    'drum_cyan': {
        'opt': 0,
        'remaining': 100,
        'newError': ''
    },
    'drum_magenta': {
        'opt': 0,
        'remaining': 100,
        'newError': ''
    },
    'drum_yellow': {
        'opt': 0,
        'remaining': 100,
        'newError': ''
    },
    'tray1': {
        'opt': 1,
        'paper_size1': 4,
        'paper_size2': 0,
        'paper_type1': 2,
        'paper_type2': 0,
        'capa': 150,
        'newError': ''
    },
    'tray2': {
        'opt': 0,
        'paper_size1': 0,
        'paper_size2': 0,
        'paper_type1': 2,
        'paper_type2': 0,
        'capa': 0,
        'newError': ''
    },
    'tray3': {
        'opt': 0,
        'paper_size1': 0,
        'paper_size2': 0,
        'paper_type1': 2,
        'paper_type2': 0,
        'capa': 0,
        'newError': ''
    },
    'tray4': {
        'opt': 0,
        'paper_size1': 0,
        'paper_size2': 0,
        'paper_type1': 2,
        'paper_type2': 0,
        'capa': 0,
        'newError': ''
    },
    'tray5': {
        'opt': 0,
        'paper_size1': 0,
        'paper_size2': 0,
        'paper_type1': 0,
        'paper_type2': 0,
        'capa': 0,
        'newError': '0'
    },
    'mp': {
        'opt': 0,
        'paper_size1': 0,
        'paper_size2': 0,
        'paper_type1': 2,
        'paper_type2': 0,
        'capa': 0,
        'newError': ''
    },
    'manual': {
        'opt': 0,
        'paper_size1': 0,
        'paper_size2': 0,
        'paper_type1': 2,
        'paper_type2': 0,
        'capa': 0,
        'newError': ''
    },
    'GXI_INTRAY_MANUALFEEDING_TRAY_SUPPORT': 0,
    'GXI_INSTALL_OPTION_MULTIBIN': 0,
    'multibin': [['Bin', 100, 0]],
    'outputTray': [[1, 100, '']],
    'capability': {
        'hdd': {
            'opt': 2,
            'capa': 0
        },
        'ram': {
            'opt': 131072,
            'capa': 131072
        },
        'scanner': {
            'opt': 0,
            'capa': 0
        }
    },
    'options': {
        'wlan': 1
    },
    'GXI_ACTIVE_ALERT_TOTAL': 0,
    'GXI_ADMIN_WUI_HAS_DEFAULT_PASS': 1,
    'GXI_SUPPORT_COLOR': 2,
    'GXI_SYS_LUI_SUPPORT': 1,
    'GXI_A3_SUPPORT': 0,
    'GXI_TRAY2_MANDATORY_SUPPORT': 0,
    'GXI_SWS_ADMIN_USE_AAA': 0,
    'GXI_TONER_BLACK_VALID': 1,
    'GXI_TONER_CYAN_VALID': 1,
    'GXI_TONER_MAGENTA_VALID': 1,
    'GXI_TONER_YELLOW_VALID': 1,
    'GXI_IMAGING_BLACK_VALID': 1,
    'GXI_IMAGING_CYAN_VALID': 1,
    'GXI_IMAGING_MAGENTA_VALID': 1,
    'GXI_IMAGING_YELLOW_VALID': 1,
    'GXI_SUPPORT_PAPER_SETTING': 1
}



"""Connect to a Samsung printer with SyncThru service."""

import demjson

import asyncio
import aiohttp
import async_timeout

from typing import Any, Dict

ENDPOINT = "/sws/app/information/home/home.json"


def construct_url(ip_address: str) -> str:
    """Construct the URL with a given IP address."""
    if 'http://' not in ip_address and 'https://' not in ip_address:
        ip_address = '{}{}'.format('http://', ip_address)
    if ip_address[-1] == '/':
        ip_address = ip_address[:-1]
    return ip_address


class SyncThru:
    """Interface to communicate with the Samsung Printer with SyncThru."""
    COLOR_NAMES = ['black', 'cyan', 'magenta', 'yellow']
    TONER = 'toner'
    DRUM = 'drum'
    TRAY = 'tray'
    OFFLINE = 'Offline'

    def __init__(self, ip, loop, session) -> None:
        """Initialize the connection to the printer."""
        self.url = construct_url(ip)
        self._loop = loop
        self._session = session
        self.data = {}  # type: Dict[str, Any]

    async def update(self) -> None:
        """Retrieve the data from the printer."""
        url = '{}{}'.format(self.url, ENDPOINT)

        try:
            async with async_timeout.timeout(5, loop=self._loop):
                response = await self._session.get(url)
            json_dict = demjson.decode(await response.text())
        except (asyncio.TimeoutError, aiohttp.ClientError):
            json_dict = {'status': {'status1': SyncThru.OFFLINE}}
        except (KeyError, ValueError):
            json_dict = {}
        self.data = json_dict

    @staticmethod
    def device_status_simple(status: str) -> str:
        """Convert the status1 field of the device status to a string."""
        return {
            '  Sleeping...   ': 'Sleeping',
            ' Ready to Copy  ': 'Ready',
            '   Warming Up   ': 'Warming up',
            SyncThru.OFFLINE: 'Offline',
        }.get(status, 'Unknown')

    def is_online(self) -> bool:
        """Return true if printer is online."""
        return (self.device_status() != SyncThru.OFFLINE
                and self.device_status() != 'Unknown')

    def model(self):
        """Return the model name of the printer."""
        try:
            return self.data.get('identity').get('model_name')
        except (KeyError, AttributeError):
            return self.device_status_simple('')

    def location(self):
        """Return the location of the printer."""
        try:
            return self.data.get('identity').get('location')
        except (KeyError, AttributeError):
            return self.device_status_simple('')

    def serial_number(self):
        """Return the serial number of the printer."""
        try:
            return self.data.get('identity').get('serial_num')
        except (KeyError, AttributeError):
            return self.device_status_simple('')

    def hostname(self):
        """Return the hostname of the printer."""
        try:
            return self.data.get('identity').get('host_name')
        except (KeyError, AttributeError):
            return self.device_status_simple('')

    def device_status(self):
        """Return the status of the device as string."""
        try:
            return self.device_status_simple(
                self.data.get('status').get('status1'))
        except (KeyError, AttributeError):
            return self.device_status_simple('')

    def capability(self) -> Dict[str, Any]:
        """Return the capabilities of the printer."""
        try:
            return self.data.get('capability', {})
        except (KeyError, AttributeError):
            return {}

    def raw(self):
        """Return all details of the printer."""
        try:
            return self.data
        except (KeyError, AttributeError):
            return {}

    def toner_status(self, filter_supported: bool = True) -> Dict[str, Any]:
        """Return the state of all toners cartridges."""
        toner_status = {}
        for color in self.COLOR_NAMES:
            try:
                toner_stat = self.data.get(
                    '{}_{}'.format(SyncThru.TONER, color), {})
                if filter_supported and toner_stat.get('opt', 0) == 0:
                    continue
                else:
                    toner_status[color] = toner_stat
            except (KeyError, AttributeError):
                toner_status[color] = {}
        return toner_status

    def input_tray_status(self,
                          filter_supported: bool = True) -> Dict[int, Any]:
        """Return the state of all input trays."""
        tray_status = {}
        for i in range(1, 5):
            try:
                tray_stat = self.data.get('{}{}'.format(SyncThru.TRAY, i), {})
                if filter_supported and tray_stat.get('opt', 0) == 0:
                    continue
                else:
                    tray_status[i] = tray_stat
            except (KeyError, AttributeError):
                tray_status[i] = {}
        return tray_status

    def output_tray_status(self) -> Dict[int, Dict[str, str]]:
        """Return the state of all output trays."""
        tray_status = {}
        try:
            tray_stat = self.data.get('outputTray', [])
            for i, stat in enumerate(tray_stat):
                tray_status[i] = {
                    'name': stat[0],
                    'capacity': stat[1],
                    'status': stat[2],
                }
        except (KeyError, AttributeError):
            tray_status = {}
        return tray_status

    def drum_status(self, filter_supported: bool = True) -> Dict[str, Any]:
        """Return the state of all drums."""
        drum_status = {}
        for color in self.COLOR_NAMES:
            try:
                drum_stat = self.data.get('{}_{}'.format(SyncThru.DRUM, color),
                                          {})
                if filter_supported and drum_stat.get('opt', 0) == 0:
                    continue
                else:
                    drum_status[color] = drum_stat
            except (KeyError, AttributeError):
                drum_status[color] = {}
        return drum_status



import os
import urllib.parse
from http.server import SimpleHTTPRequestHandler, HTTPServer
try:
    from http import HTTPStatus
except ImportError:
    # Backwards compatability
    import http.client as HTTPStatus
import posixpath
from pathlib import Path

SERVER_DIR = Path(__file__).parent or Path('.')

SyncThruServer = HTTPServer


class SyncThruRequestHandler(SimpleHTTPRequestHandler):
    def translate_path(self, path):
        """Translate a /-separated PATH to the local filename syntax.

        Components that mean special things to the local file system
        (e.g. drive or directory names) are ignored.  (XXX They should
        probably be diagnosed.)

        only slightly changed method of the standard library
        """
        # abandon query parameters
        path = path.split('?', 1)[0]
        path = path.split('#', 1)[0]
        # Don't forget explicit trailing slash when normalizing. Issue17324
        trailing_slash = path.rstrip().endswith('/')
        try:
            path = urllib.parse.unquote(path, errors='surrogatepass')
        except UnicodeDecodeError:
            path = urllib.parse.unquote(path)
        path = posixpath.normpath(path)
        words = path.split('/')
        words = filter(None, words)
        path = str(SERVER_DIR.absolute())
        for word in words:
            if os.path.dirname(word) or word in (os.curdir, os.pardir):
                # Ignore components that are not a simple file/directory name
                continue
            path = os.path.join(path, word)
        if trailing_slash:
            path += '/'
        return path

    def send_error(self, code, message=None, explain=None):
        """
        Send syncthru error page
        :param code:
        :param message:
        :param explain:
        :return:
        """
        self.log_error("code %d, message %s", code, message)
        self.send_response(code, message)
        self.send_header('Connection', 'close')

        # Message body is omitted for cases described in:
        #  - RFC7230: 3.3. 1xx, 204(No Content), 304(Not Modified)
        #  - RFC7231: 6.3.6. 205(Reset Content)
        body = None
        if (code >= 200 and
                code not in (HTTPStatus.NO_CONTENT, HTTPStatus.RESET_CONTENT,
                             HTTPStatus.NOT_MODIFIED)):
            # HTML encode to prevent Cross Site Scripting attacks
            # (see bug #1100201)
            # Specialized error method for fronius
            with SERVER_DIR.joinpath(".error.html").open('rb') as file:
                body = file.read()
            self.send_header("Content-Type", self.error_content_type)
            self.send_header('Content-Length', int(len(body)))
        self.end_headers()

        if self.command != 'HEAD' and body:
            self.wfile.write(body)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# general requirements
import unittest
from .test_structure.server_control import Server
from .test_structure.syncthru_mock_server import SyncThruServer, SyncThruRequestHandler

# For the server in this case
import time

# For the tests
import aiohttp
import asyncio
from pysyncthru import SyncThru
from .web_raw.web_state import RAW

ADDRESS = 'localhost'


class SyncthruWebTest(unittest.TestCase):

    server = None
    server_control = None
    port = 0
    url = 'http://localhost:80'
    syncthru = None

    def setUp(self):
        # Create an arbitrary subclass of TCP Server as the server to be started
        # Here, it is an Simple HTTP file serving server
        handler = SyncThruRequestHandler

        max_retries = 10
        r = 0
        while not self.server:
            try:
                # Connect to any open port
                self.server = SyncThruServer((ADDRESS, 0), handler)
            except OSError:
                if r < max_retries:
                    r += 1
                else:
                    raise
                time.sleep(1)

        self.server_control = Server(self.server)
        self.port = self.server_control.get_port()
        self.url = "{}:{}".format(ADDRESS, self.port)
        # Start test server before running any tests
        self.server_control.start_server()

        async def fetch():
            async with aiohttp.ClientSession() as session:
                self.syncthru = SyncThru(self.url, loop, session)
                await self.syncthru.update()

        loop = asyncio.get_event_loop()
        loop.run_until_complete(fetch())

    def test_online(self):
        self.assertTrue(self.syncthru.is_online())

    def test_status_sleeping(self):
        self.assertEqual(self.syncthru.device_status(), 'Sleeping')

    def test_model(self):
        self.assertEqual(self.syncthru.model(), RAW['identity']['model_name'])

    def test_toner_filter(self):
        self.assertEqual(
            self.syncthru.toner_status(True),
            {'black': {
                'opt': 1,
                'remaining': 58,
                'cnt': 229,
                'newError': ''
            }})

    def test_toner_no_filter(self):
        empty = {'opt': 0, 'remaining': 0, 'cnt': 0, 'newError': ''}
        self.assertEqual(
            self.syncthru.toner_status(False), {
                'yellow': empty,
                'magenta': empty,
                'cyan': empty,
                'black': {
                    'opt': 1,
                    'remaining': 58,
                    'cnt': 229,
                    'newError': ''
                }
            })

    def test_input_tray_filter(self):
        self.assertEqual(
            self.syncthru.input_tray_status(True), {
                1: {
                    'capa': 150,
                    'newError': '',
                    'opt': 1,
                    'paper_size1': 4,
                    'paper_size2': 0,
                    'paper_type1': 2,
                    'paper_type2': 0
                }
            })

    def test_input_tray_no_filter(self):
        self.assertEqual(
            self.syncthru.input_tray_status(False), {
                1: {
                    'capa': 150,
                    'newError': '',
                    'opt': 1,
                    'paper_size1': 4,
                    'paper_size2': 0,
                    'paper_type1': 2,
                    'paper_type2': 0
                },
                2: {
                    'capa': 0,
                    'newError': '',
                    'opt': 0,
                    'paper_size1': 0,
                    'paper_size2': 0,
                    'paper_type1': 2,
                    'paper_type2': 0
                },
                3: {
                    'capa': 0,
                    'newError': '',
                    'opt': 0,
                    'paper_size1': 0,
                    'paper_size2': 0,
                    'paper_type1': 2,
                    'paper_type2': 0
                },
                4: {
                    'capa': 0,
                    'newError': '',
                    'opt': 0,
                    'paper_size1': 0,
                    'paper_size2': 0,
                    'paper_type1': 2,
                    'paper_type2': 0
                }
            })

    def test_output_tray(self):
        self.assertEqual(self.syncthru.output_tray_status(),
                         {0: {
                             'capacity': 100,
                             'name': 1,
                             'status': ''
                         }})

    def test_drum_status_filter(self):
        self.assertEqual(self.syncthru.drum_status(True), {})

    def test_drum_status_no_filter(self):
        self.assertEqual(
            self.syncthru.drum_status(False), {
                'black': {
                    'newError': '',
                    'opt': 0,
                    'remaining': 0
                },
                'cyan': {
                    'newError': '',
                    'opt': 0,
                    'remaining': 100
                },
                'magenta': {
                    'newError': '',
                    'opt': 0,
                    'remaining': 100
                },
                'yellow': {
                    'newError': '',
                    'opt': 0,
                    'remaining': 100
                }
            })

    def test_location(self):
        self.assertEqual(self.syncthru.location(), RAW['identity']['location'])

    def test_serial_number(self):
        self.assertEqual(self.syncthru.serial_number(),
                         RAW['identity']['serial_num'])

    def test_hostname(self):
        self.assertEqual(self.syncthru.hostname(),
                         RAW['identity']['host_name'])

    def test_cap(self):
        self.assertEqual(self.syncthru.capability(), RAW['capability'])

    def tearDown(self):
        self.server_control.stop_server()
        pass


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# coding=utf-8
from setuptools import setup, find_packages

from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

short_description = '{}'.format(
    'Automated JSON API based communication with Samsung SyncThru Web Service')

setup(
    name='PySyncThru',
    version='0.4',
    description=short_description,
    author='Niels Mündler',
    author_email='n.muendler@web.de',
    url='https://github.com/nielstron/pysyncthru/',
    py_modules=['pysyncthru'],
    packages=find_packages(),
    package_data={'pysyncthru.tests.test_structure': ['sws/app/information/home/home.json', '*.html']},
    install_requires=[
        'demjson',
        'aiohttp',
        'async_timeout',
    ],
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Object Brokering',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='python syncthru json api samsung printer',
    python_requires='>=3',
    test_suite='pysyncthru.tests',
)
